
;================================================================================
;x PROC CarregaPaleta                                                           x
;x                                                                              x
;x Descrio: Procedimento que carrega uma paleta de cores                      x
;x            na sua varivel correspondente.                                   x
;x                                                                              x
;x Parametros:                                                                  x
;x DS:DX - Nome do arquivo da paleta                                            x
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
CarregaPaleta PROC NEAR

              ;Empilha os registradores
              push ax
              push bx
              push cx
              push dx
              push ds
              push es
              push di
              push si

              ;Faz com que ES aponte para o segmento de dados
              mov ax,@data
              mov es,ax   ;trocar depois para ds, pra ver se funciona

              ;Passa o endereo do arquivo da paleta para DI
              mov di, offset palleteBuffer
              mov cx, 768
              ;L o arquivo
              call ReadFile

              ;Ativa escrita da paleta (Porta 3C8H)

              ;Avisa a interface de video que a paleta vai ser modificada
              mov dx,3C8h
              mov al,0
              out dx,al

              ;Incrementa o valor de dx para 3C9H, para modificar a paleta
              inc dx

              ;Manda 768 bytes do buffer para a paleta
              ;outsb envia o contedo de si para o endereo da porta contido em dx
              mov si, offset palleteBuffer
              mov ax, seg palleteBuffer
              mov ds,ax
              mov cx,768
              rep outsb


              ; Desempilha Registradores
              pop si
              pop di
              pop es
              pop ds
              pop dx
              pop cx
              pop bx
              pop ax
              ret

CarregaPaleta ENDP
;================================================================================
;  FIM do Procedimento CarregaPaleta
;================================================================================
    



;================================================================================
;  Procedimento ReadFile
;  Copia Arquivo para Buffer
;  Parametros
;    DS:DX   Nome do Arquivo
;    ES:DI   Buffer
;    CX      Tamanho
;================================================================================

ReadFile PROC NEAR
                                           ;Empilha registradores
                   pusha
                   push ds
                   push es
                                           ; Abre o arquivo e verifica erros
                   mov ax, 3D00h
                   int 21h
                   jc  ErrorOpenFile
                                           ;Copia o Handle de AX para BX
                   mov bx, ax
                                           ;Copia o conteudo do arquivo para o buffer
                   mov dx, es
                   mov ds, dx
                   mov dx, di
                   mov ah, 3Fh
                   int 21h
                   jc  ErrorOpenFile
                                           ;Fecha o arquivo
                   mov ah, 3Eh
                   int 21h
                   jc  ErrorOpenFile

                                           ;Finaliza o procedimento
    RFExit:
                                           ;Desempilha registradores
                   pop es
                   pop ds
                   popa
                   ret
                                           ;Tratamento de erros
   ErrorOpenFile1:

                   mov dx, offset msg1
                   mov ax, 03h             ; mudar para mostrar figura
                   int 10h
                                           ;Exibe a string (DS:DX) na tela
                   mov ah, 09h
                   int 21h
                                           ;Retorna o controle ao SO
                   mov ax, 4C01h
                   int 21h

    ErrorOpenFile:
                                           ;Verifica se foi FileNotFoundError
                   cmp ax, 02h
                   je  ErrorFileNotFound
                                           ;DS:DX aponta para mensagem de erro
                   mov dx, offset msg1
                   jmp ErrorExit
                                           ;Trata o erro de FileNotFoundError
    ErrorFileNotFound:
                                           ;Verifica se foi o arquivo de Recordes
                   cmp dx, offset scoreFile
                   je  CreateScoreFile
                                           ;DS:DX aponta para mensagem de erro
                   mov dx, offset msg2
                                           ;Finaliza o tratamento de erros
    ErrorExit:
                                           ;Retorna ao modo texto
                   mov ax, 03h
                   int 10h
                                           ;Exibe a string (DS:DX) na tela
                   mov ah, 09h
                   int 21h
                                           ;Retorna o controle ao SO
                   mov ax, 4C01h
                   int 21h
                                           ;Cria o arquivo de Recordes
    CreateScoreFile:
                                           ;Cria o arquivo
                   mov cx, 00h
                   mov ah, 3Ch
                   int 21h
                                           ;Fecha o arquivo
                   mov bx, ax
                   mov ah, 3Eh
                   int 21h
                                           ;Finaliza o procedimento
                   jmp RFExit


ReadFile        ENDP
;================================================================================
;  FIM do Procedimento ReadFile
;================================================================================
    

;================================================================================
;x Proc DesenhaTela
;x
;x Descrio: Copia uma imagem de um Arquivo para o Video
;x
;x Parametros
;x   DS:DX   Nome do Arquivo (ASCIIZ)
;================================================================================
DesenhaTela PROC NEAR

;Empilha Registradores
push ax
push bx
push cx
push dx
push ds
push es

;Abre o Arquivo (3D) como somente leitura (00)
mov ah, 3Dh
;Seta somente leitura
mov al, 00h
int 21h

;Copia o Handle do Arquivo de AX para BX que sera usado na proxima int 21h
mov bx, ax

;DS:DX Aponta para a Memoria de Video
mov ax, 0A000h
mov ds, ax
mov dx, 0

;L a partir do arquivo aberto e o copia para a Memoria de Video
;cx - Numero de iteraes - 64000 (320x200)
mov cx, 64000
mov ah, 3Fh
int 21h

;Fecha o Arquivo
mov ah, 3Eh
int 21h

;Desempilha Registradores
pop es
pop ds
pop dx
pop cx
pop bx
pop ax
ret
;Fim do procedimento DesenhaTela
DesenhaTela     ENDP

;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
; PROCEDIMENTO ATRASO2
; Da um  intervalo maior entre telas
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
atraso2 PROC NEAR
	  pusha
          mov     ah,0
          int     01ah
          add     dl,50h
          mov     bl,dl
wait2:    mov     ah,0
          int     01ah
          cmp     dl,bl
          jnz     wait2
          popa
          ret
atraso2 ENDP
; xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
; PROCEDIMENTO ATRASO
; Da um  intervalo menor entre telas
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
atraso PROC NEAR
	  pusha
          mov     ah,0
          int     01ah
          add     dl,2h
          mov     bl,dl
wait3:    mov     ah,0
          int     01ah
          cmp     dl,bl
          jnz     wait3
          popa
          ret
atraso ENDP
; xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
